import Foundation
import Combine
import SwiftUI

class AuthManager: ObservableObject {
    @Published var isAuthenticated = false
    @Published var currentUser: User?
    @Published var isLoading = false
    @Published var errorMessage: String?
    
    private var cancellables = Set<AnyCancellable>()
    private let apiClient = APIClient.shared
    
    init() {
        checkAuthStatus()
    }
    
    private func checkAuthStatus() {
        isLoading = true
        
        apiClient.verify()
            .receive(on: DispatchQueue.main)
            .sink(
                receiveCompletion: { [weak self] completion in
                    self?.isLoading = false
                    if case .failure = completion {
                        self?.isAuthenticated = false
                        self?.currentUser = nil
                    }
                },
                receiveValue: { [weak self] user in
                    self?.isAuthenticated = true
                    self?.currentUser = user
                }
            )
            .store(in: &cancellables)
    }
    
    func login(identifier: String, password: String) {
        isLoading = true
        errorMessage = nil
        
        apiClient.login(identifier: identifier, password: password)
            .receive(on: DispatchQueue.main)
            .sink(
                receiveCompletion: { [weak self] completion in
                    self?.isLoading = false
                    if case .failure(let error) = completion {
                        self?.errorMessage = self?.getErrorMessage(from: error)
                    }
                },
                receiveValue: { [weak self] response in
                    self?.isAuthenticated = true
                    self?.currentUser = User(did: response.did, handle: response.handle)
                }
            )
            .store(in: &cancellables)
    }
    
    func logout() {
        isLoading = true
        
        apiClient.logout()
            .receive(on: DispatchQueue.main)
            .sink(
                receiveCompletion: { [weak self] _ in
                    self?.isLoading = false
                    self?.isAuthenticated = false
                    self?.currentUser = nil
                    UserDefaults.standard.removeObject(forKey: "authToken")
                },
                receiveValue: { _ in }
            )
            .store(in: &cancellables)
    }
    
    private func getErrorMessage(from error: APIError) -> String {
        switch error {
        case .unauthorized:
            return "認証情報が正しくありません"
        case .networkError:
            return "ネットワークエラーが発生しました"
        default:
            return "エラーが発生しました"
        }
    }
}