import React, { useState, useEffect } from 'react';
import { Card } from './components/Card';
import { GachaAnimation } from './components/GachaAnimation';
import { Login } from './components/Login';
import { cardApi } from './services/api';
import { authService, User } from './services/auth';
import { Card as CardType, CardDrawResult } from './types/card';
import './App.css';

function App() {
  const [isDrawing, setIsDrawing] = useState(false);
  const [currentDraw, setCurrentDraw] = useState<CardDrawResult | null>(null);
  const [userCards, setUserCards] = useState<CardType[]>([]);
  const [error, setError] = useState<string | null>(null);
  const [user, setUser] = useState<User | null>(null);
  const [showLogin, setShowLogin] = useState(false);
  const [isLoading, setIsLoading] = useState(true);

  useEffect(() => {
    // Check if user is logged in
    authService.verify().then(verifiedUser => {
      if (verifiedUser) {
        setUser(verifiedUser);
        loadUserCards(verifiedUser.did);
      }
      setIsLoading(false);
    });
  }, []);

  const loadUserCards = async (did: string) => {
    try {
      const cards = await cardApi.getUserCards(did);
      setUserCards(cards);
    } catch (err) {
      console.error('Failed to load cards:', err);
    }
  };

  const handleLogin = (did: string, handle: string) => {
    setUser({ did, handle });
    setShowLogin(false);
    loadUserCards(did);
  };

  const handleLogout = async () => {
    await authService.logout();
    setUser(null);
    setUserCards([]);
  };

  const handleDraw = async (isPaid: boolean = false) => {
    if (!user) {
      setShowLogin(true);
      return;
    }

    setIsDrawing(true);
    setError(null);

    try {
      const result = await cardApi.drawCard(user.did, isPaid);
      setCurrentDraw(result);
    } catch (err) {
      setError('カードの抽選に失敗しました');
      setIsDrawing(false);
    }
  };

  const handleAnimationComplete = () => {
    if (currentDraw) {
      setUserCards([...userCards, currentDraw.card]);
      setCurrentDraw(null);
      setIsDrawing(false);
    }
  };

  if (isLoading) {
    return (
      <div className="app">
        <div className="loading">Loading...</div>
      </div>
    );
  }

  return (
    <div className="app">
      <header className="app-header">
        <h1>ai.card</h1>
        <p>atprotoベースカードゲーム</p>
        <div className="user-info">
          {user ? (
            <>
              <span className="user-handle">@{user.handle}</span>
              <button onClick={handleLogout} className="logout-button">
                ログアウト
              </button>
            </>
          ) : (
            <button onClick={() => setShowLogin(true)} className="login-button">
              ログイン
            </button>
          )}
        </div>
      </header>

      <main className="app-main">
        <section className="gacha-section">
          <h2>カードを引く</h2>
          <div className="gacha-buttons">
            <button 
              onClick={() => handleDraw(false)} 
              disabled={isDrawing}
              className="gacha-button"
            >
              通常ガチャ
            </button>
            <button 
              onClick={() => handleDraw(true)} 
              disabled={isDrawing}
              className="gacha-button gacha-button-premium"
            >
              プレミアムガチャ
            </button>
          </div>
          {error && <p className="error">{error}</p>}
        </section>

        <section className="collection-section">
          <h2>コレクション</h2>
          <div className="card-grid">
            {userCards.map((card, index) => (
              <Card key={index} card={card} />
            ))}
          </div>
          {userCards.length === 0 && (
            <p className="empty-message">
              {user ? 'まだカードを持っていません' : 'ログインしてカードを集めよう'}
            </p>
          )}
        </section>
      </main>

      {currentDraw && (
        <GachaAnimation
          card={currentDraw.card}
          animationType={currentDraw.animation_type}
          onComplete={handleAnimationComplete}
        />
      )}

      {showLogin && (
        <Login
          onLogin={handleLogin}
          onClose={() => setShowLogin(false)}
        />
      )}
    </div>
  );
}

export default App;